const css = require('css')
const { ServerStyleSheet } = require('styled-components')
const StyleSheet = require('styled-components/lib/models/StyleSheet')

const isOverV2 = () => Boolean(ServerStyleSheet)

const isServer = () => typeof document === 'undefined'

const resetStyleSheet = () => {
  if (isOverV2()) {
    StyleSheet.default.reset(isServer())
  }
}

const getHTML = () =>
  isServer()
    ? new ServerStyleSheet().getStyleTags()
    : StyleSheet.default.instance.toHTML()

const extract = regex => {
  let style = ''
  let matches

  while ((matches = regex.exec(getHTML())) !== null) {
    style += `${matches[1]} `
  }

  return style.trim()
}

const getStyle = () => extract(/<style[^>]*>([^<]*)</g)

const getCSS = () => {
  const style = isOverV2()
    ? getStyle()
    : StyleSheet.rules().map(rule => rule.cssText).join('\n')

  return css.parse(style)
}

const getClassNames = () => extract(/data-styled-components="([^"]*)"/g)

const getComponentIDs = () => extract(/sc-component-id: ([^\\*\\/]*) \*\//g)

const getHashes = () => `${getClassNames()} ${getComponentIDs()}`

module.exports = {
  resetStyleSheet,
  getCSS,
  getHashes,
}
